!! Copyright 2012, Marco Spadini
!! e-mail: marco.spadini@unifi.it, web: http://www.dma.unifi.it/~spadini
!!
!! Program for the approximate search of an example of 4 matrices 2x2 with positive 
!! determinant, defining a piecewise continuous map, and whose convex combination 
!! has negative determinant. Geometric considerations allow for the search to be 
!! restricted to matrices of a particular form. Matrix A1 is the l'identity.
!!
!! This program has been used for Example 4.4 in the paper:
!!
!! L.Poggiolini — M.Spadini, Local inversion of planar maps with nice nondifferentiability 
!! structure. To appear in Adv. Nonlin. Studies
!!
!! As the approximate example was available, it was easier to find a "neater" one, with 
!! nice fractions instead of nasty floating point numbers (which are themselves fractions, 
!! after all), using pen and paper!
!!
pure function det(M)
  real :: det
  real, dimension(2,2), intent(IN) :: M
  det = M(1,1)*M(2,2)-M(1,2)*M(2,1)
  end function det


program controesempio
  implicit none
  interface
    pure function det(M)
    real :: det
    real, dimension(2,2), intent(IN) :: M
    end function det
  end interface
  integer, parameter :: nit = 100
  real, dimension(2,2) :: A1,A2,A3,A4,M
  real :: l1,l2,l3,l4
  real :: alpha, d, b, gamma
  real :: e,f,g,h
  integer :: i,j,k
  integer :: s,p,q,r
  integer :: contatore = 0

  A1= reshape((/1.,0.,0.,1./),(/2,2/))

  cicloGrosso: do s=1,nit
    alpha=float(s)/10.
    do p=-nit,nit
      gamma=float(p)/10.
      do q=-nit,nit
        b=float(q)/10.
        do r=1,nit
          d=float(r)/10.

          e = (1.-b+alpha)/2.
          f = (alpha+b-1.)/2.
          g = (gamma+1.-d)/2.
          h = (gamma+1.+d)/2.

          contatore = contatore + 1

          if (e*h-f*g .gt. 0.) then
            A2= reshape((/alpha,gamma,0.,1./),(/2,2/))
            A3= reshape((/e,g,f,h/),(/2,2/))
            A4= reshape((/1.,0.,b,d/),(/2,2/))

            do i=0,nit
              l1=float(i)/nit
              do j=0,nit-i
                l2=float(j)/nit
                do k=0,nit-(j+i)
                  l3=float(k)/nit
                  l4=1.-l1-l2-l3
                  M=l1*A1+l2*A2+l3*A3+l4*A4
                  contatore = contatore + 1
                  if (det(M) .lt. -0.5) then
                    print*,
                    print*, 'Trovato! ...Valori parametri:'
                    print '(A,F10.8)', 'l1=',l1,'l2=',l2,'l3=',l3,'l4=',l4
                    print*,
                    print*, 'Matrici A1...A4 (per colonne):'
                    print*,'A1=',A1(1,1),a1(1,2),a1(2,1),a1(2,2)
                    print*,'A2=',A2(1,1),a2(1,2),a2(2,1),a2(2,2)
                    print*,'A3=',A3(1,1),a3(1,2),a3(2,1),a3(2,2)
                    print*,'A4=',A4(1,1),a4(1,2),a4(2,1),a4(2,2)
                    print*,
                    print*,"M = l1*A1+l2*A2+l3*A3+l4*A4 ="
                    print*,m(1,1),m(1,2)
                    print*,m(2,1),m(2,2)
                    print*,
                    print*, 'Determinante di M= ', det(M)
                    print*,
                    print*, 'Ho esaminato', contatore,' matrici.'
                    exit cicloGrosso
                    end if
                  end do
                end do
              end do
            end if
          end do
        end do
      end do
    end do cicloGrosso

end program controesempio
